]> git.cworth.org Git - zombocom-ai/blobdiff - index.js
Replay previous images to a client when it first connects
[zombocom-ai] / index.js
index 9279c877250c9e6055d142f9839dddd22dae775d..0ee5e4359255f4afe794114cd043950d0da2c85e 100644 (file)
--- a/index.js
+++ b/index.js
@@ -1,5 +1,8 @@
 const fs = require('fs');
 
+const util = require('util');
+const execFile = util.promisify(require('child_process').execFile);
+
 const express = require('express');
 const app = express();
 const http = require('http');
@@ -8,11 +11,23 @@ const { Server } = require("socket.io");
 const io = new Server(server);
 const port = 2122;
 
-comments = [];
+const python_path = '/usr/bin/python3'
+const generate_image_script = '/home/cworth/src/zombocom-ai/generate-image.py'
+const state_file = 'zombocom-state.json'
+
+var state;
+
+// Load comments at server startup
+fs.readFile(state_file, (err, data) => {
+    if (err)
+        state = { images: [], comments: [] };
+    else
+        state = JSON.parse(data);
+});
 
 // Save comments when server is shutting down
 function cleanup() {
-    fs.writeFileSync('zombocom-state.json', JSON.stringify(comments), (error) => {
+    fs.writeFileSync('zombocom-state.json', JSON.stringify(state), (error) => {
         if (error)
             throw error;
     })
@@ -32,14 +47,46 @@ app.get('/index.html', (req, res) => {
 });
 
 io.on('connection', (socket) => {
-    // Replay old comments to a newly-joining client
-    comments.forEach((comment) => {
+
+    // Replay old comments and images to a newly-joining client
+    state.comments.forEach((comment) => {
         socket.emit('comment', comment)
     });
+    state.images.forEach((image) => {
+        socket.emit('image', image)
+    });
+
     // When any client comments, send that to all clients (including sender)
     socket.on('comment', (comment) => {
         io.emit('comment', comment);
-        comments.push(comment);
+        state.comments.push(comment);
+    });
+
+    // Generate an image when requested
+    socket.on('generate', (request) => {
+        console.log(`Generating image with code=${request['code']} and prompt=${request['prompt']}`);
+        async function generate_image(code, prompt) {
+            var promise;
+            if (code) {
+                promise = execFile(python_path, [generate_image_script, `--seed=${code}`, prompt])
+            } else {
+                promise = execFile(python_path, [generate_image_script, prompt])
+            }
+            const child = promise.child;
+            child.stdout.on('data', (data) => {
+                const images = JSON.parse(data);
+                images.forEach((image) => {
+                    console.log(`Emitting image to clients: ${image}`);
+                    io.emit('image', image);
+                    state.images.push(image);
+                });
+            });
+            child.stderr.on('data', (data) => {
+                console.log("Error occurred during generate-image: " + data);
+            });
+            const { stdout, stderr } = await promise;
+        }
+        generate_image(request['code'], request['prompt']);
     });
 });